/* ---------------------------------------------------------------------------
   commonc++ - A C++ Common Class Library
   Copyright (C) 2005-2012  Mark A Lindner

   This file is part of commonc++.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   ---------------------------------------------------------------------------
*/

#ifndef __ccxx_Digest_hxx
#define __ccxx_Digest_hxx

#include <commonc++/Common.h++>
#include <commonc++/IOException.h++>

namespace ccxx {

/** An abstract base class for message digests.
 *
 * @author Mark Lindner
 */
class COMMONCPP_API Digest
{
  public:

  /** Destructor. */
  virtual ~Digest() throw();

  /** Return the size of the digest generated by this class, in bytes. */
  virtual size_t size() const throw() = 0;

  /** Update the digest with more data.
   *
   * @param buf A pointer to the data.
   * @param len The length of the data.
   *
   * @throw IOException If the maximum length of data that this class can
   * compute a digest for would be exceeded by this call.
   */
  virtual void update(const byte_t *buf, size_t len) throw(IOException) = 0;

  /** Finish generating the digest.
   *
   * @param digest The array into which to store the digest. This array
   * must be at least the length returned by <code>size()</code>.
   */
  virtual void finish(byte_t *digest) throw() = 0;

  /** Reset the object for a new digest. */
  virtual void reset() throw() = 0;
};

}; // namespace ccxx

#endif // __ccxx_Digest_hxx
